import re

from .exception import SakuraException
from . import fields
from .log import logger

class SqlUtil:
    @staticmethod
    def get_fields(fields):
        if not fields:
            return '*'

        if isinstance(fields, (list, tuple)):
            return f"`{'`, `'.join(fields)}`"

        raise SakuraException('fields error')

    @staticmethod
    def get_field_value(field_value):
        if field_value and isinstance(field_value, dict):
            _args = []
            fields = []
            for i in field_value.items():
                fields.append(f'`{i[0]}` = %s')
                _args.append(i[1])
            _field_value = ', '.join(fields)
            return _field_value, _args
        raise SakuraException('field_value is empty')

    @staticmethod
    def get_where(cond=None):
        """
        cond:[and_cond,or_cond]
        cond:[#or
            [#and
                ['id','=',1],
                ['status','=',1],
            ],
            [
                ...
            ]

        ]
        """
        if not cond:
            return '', []
        args = []
        for i in cond:
            args.extend(j[2] for j in i)
        where = 'WHERE ' + ' OR '.join([' AND '.join([f'`{j[0]}` {j[1]} %s' for j in i]) for i in cond])
        return where, args

    @staticmethod
    def get_groupby(group_by):
        if not group_by:
            return ''

        if isinstance(group_by, (list, tuple)):
            return f'GROUP BY `{"`, `".join(group_by)}`'

        raise SakuraException('group_by error')

    @staticmethod
    def get_orderby(order_by):
        if not order_by:
            return ''

        if isinstance(order_by, (list, tuple)):
            return f'ORDER BY `{"`, `".join(order_by)}`'

        raise SakuraException('order_by error')

    @staticmethod
    def get_limit(limit):
        if not limit:
            return ''

        if isinstance(limit, int):
            if limit == -1:
                return ''
            return f'LIMIT {limit}'

        if isinstance(limit, (list, tuple)):
            return f'LIMIT {limit[0]},{limit[1]}'

        raise SakuraException('limit error')

    @staticmethod
    def format_sql(sql):
        return ' '.join([i for i in sql if i])

    @staticmethod
    def getField(field_type: str):
        if field_type in fields.string_list or field_type in fields.datetime_list:
            field = fields.BaseString
        elif field_type in fields.int_list:
            field = fields.BaseInt
        elif field_type in fields.float_list:
            field = fields.BaseFloat
        else:
            logger.warning(f'This type:{field_type} is not supported,use default type:varchar')
            field = fields.BaseString
        return field



from .exception import SakuraException
from .meta import ModelMetaclass
from .log import logger

class Model(dict, metaclass=ModelMetaclass):

    def __init__(self, **kw):
        super().__init__(**kw)
        self.modify = False

    def __getattr__(self, key):
        try:
            if key in self.__mappings__:
                return self.__mappings__[key].convert(self[key])  # .value()
            else:
                return self.__dict__[key]
        except KeyError:
            raise AttributeError(r"'Model' object has no field '%s'" % key)

    def __setattr__(self, key, value):
        if key in self.__mappings__:
            self.modify = True
            if self.__mappings__[key].validate():
                self[key] = value
            else:
                logger.error('data validation failed')
        else:
            self.__dict__[key] = value

    def Create(self):
        field_value = dict(self)
        id = self.connection.insert(self.__class__, field_value)
        self[self.__primary__] = id

    def Update(self):
        if not self.modify:
            return
        if self.__primary__ not in self:
            raise SakuraException('primary key is empty')
        cond = [[
            [self.__primary__, '=', self[self.__primary__]]
        ]]
        field_value = dict(self)
        field_value.pop(self.__primary__)
        return self.connection.update(self.__class__, field_value, cond)

    def Delete(self):
        if self.__primary__ not in self:
            raise SakuraException('primary key is empty')
        cond = [[
            [self.__primary__, '=', self[self.__primary__]]
        ]]
        return self.connection.delete(self.__class__, cond)

    def Get(self):
        cond = [[
            [k, '=', v] for k, v in self.items()
        ]]
        info = self.connection.select_one(self.__class__, cond)
        self.update(info)

    @classmethod
    def Fetch(cls, cond=None, group_by=None, order_by=None, limit=100, fields=None):
        return cls.connection.select(cls, cond, group_by, order_by, limit, fields)



from .fields import Field


class ModelMetaclass(type):
    def __new__(mcs, name, bases, attrs):
        if name == 'Model':
            return type.__new__(mcs, name, bases, attrs)
        mappings = dict()
        pk = []
        obj = None
        for k, v in attrs.items():
            if isinstance(v, Field):
                if v.primary_key:
                    pk = k
                mappings[k] = v
            elif k == 'connection':
                obj = v
        for k in mappings.keys():
            attrs.pop(k)
        attrs['__connection__'] = obj
        attrs['__primary__'] = pk
        attrs['__mappings__'] = mappings  # 保存属性和列的映射关系
        attrs['__table__'] = name.lower()  # 假设表名和类名一致
        return type.__new__(mcs, name, bases, attrs)
